@@ -44,7 +44,7 @@ void   tracker_data_backup_restore     (GFile                     *journal,
                                         TrackerDataBackupFinished  callback,
                                         gpointer                   user_data,
                                         GDestroyNotify             destroy,
-                                        const gchar               *test_schema);
+                                        const gchar              **test_schema);
 
 G_END_DECLS
 
