@@ -57,6 +57,11 @@ typedef enum {
 	DATA_FORMAT_GRAPH            = 1 << 3
 } DataFormat;
 
+typedef enum {
+	TRANSACTION_FORMAT_DATA      = 1 << 0,
+	TRANSACTION_FORMAT_ONTOLOGY  = 1 << 1,
+} TransactionFormat;
+
 static struct {
 	gchar *filename;
 	GMappedFile *file;
@@ -193,7 +198,7 @@ tracker_db_journal_error_quark (void)
 }
 
 gboolean
-tracker_db_journal_init (const gchar *filename)
+tracker_db_journal_init (const gchar *filename, gboolean truncate)
 {
 	gchar *directory;
 	struct stat st;
@@ -238,6 +243,14 @@ tracker_db_journal_init (const gchar *filename)
 
 	mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP;
 	flags = O_WRONLY | O_APPEND | O_CREAT;
+	if (truncate) {
+		/* existing journal contents are invalid: reindex where journal
+		 * does not even contain a single valid entry
+		 *
+		 * or should be ignored: only for test cases
+		 */
+		flags |= O_TRUNC;
+	}
 	writer.journal = g_open (writer.journal_filename, flags, mode);
 
 	if (writer.journal == -1) {
@@ -268,7 +281,7 @@ tracker_db_journal_init (const gchar *filename)
 		writer.cur_block[4] = 'g';
 		writer.cur_block[5] = '\0';
 		writer.cur_block[6] = '0';
-		writer.cur_block[7] = '1';
+		writer.cur_block[7] = '2';
 
 		if (!write_all_data (writer.journal, writer.cur_block, 8)) {
 			g_free (writer.journal_filename);
@@ -332,7 +345,7 @@ tracker_db_journal_start_transaction (time_t time)
 
 	/* Leave space for size, amount and crc
 	 * Check and keep in sync the offset variable at
-	 * tracker_db_journal_commit_transaction too */
+	 * tracker_db_journal_commit_db_transaction too */
 
 	memset (writer.cur_block, 0, size);
 
@@ -344,6 +357,44 @@ tracker_db_journal_start_transaction (time_t time)
 	cur_setnum (writer.cur_block, &writer.cur_pos, time);
 	writer.cur_block_len += sizeof (gint32);
 
+	/* Add format */
+	cur_block_maybe_expand (sizeof (gint32));
+	cur_setnum (writer.cur_block, &writer.cur_pos, TRANSACTION_FORMAT_DATA);
+	writer.cur_block_len += sizeof (gint32);
+
+	return TRUE;
+}
+
+
+gboolean
+tracker_db_journal_start_ontology_transaction (time_t time)
+{
+	guint size;
+
+	g_return_val_if_fail (writer.journal > 0, FALSE);
+
+	size = sizeof (guint32) * 3;
+	cur_block_maybe_expand (size);
+
+	/* Leave space for size, amount and crc
+	 * Check and keep in sync the offset variable at
+	 * tracker_db_journal_commit_db_transaction too */
+
+	memset (writer.cur_block, 0, size);
+
+	writer.cur_pos = writer.cur_block_len = size;
+	writer.cur_entry_amount = 0;
+
+	/* add timestamp */
+	cur_block_maybe_expand (sizeof (gint32));
+	cur_setnum (writer.cur_block, &writer.cur_pos, time);
+	writer.cur_block_len += sizeof (gint32);
+
+	/* Add format */
+	cur_block_maybe_expand (sizeof (gint32));
+	cur_setnum (writer.cur_block, &writer.cur_pos, TRANSACTION_FORMAT_ONTOLOGY);
+	writer.cur_block_len += sizeof (gint32);
+
 	return TRUE;
 }
 
@@ -552,7 +603,7 @@ tracker_db_journal_truncate (gsize new_size)
 }
 
 gboolean
-tracker_db_journal_commit_transaction (void)
+tracker_db_journal_commit_db_transaction (void)
 {
 	guint32 crc;
 	guint begin_pos;
@@ -658,7 +709,7 @@ tracker_db_journal_reader_init (const gchar *filename)
 		return FALSE;
 	}
 
-	if (memcmp (reader.current, "trlog\00001", 8)) {
+	if (memcmp (reader.current, "trlog\00002", 8)) {
 		tracker_db_journal_reader_shutdown ();
 		return FALSE;
 	}
@@ -779,11 +830,13 @@ tracker_db_journal_reader_next (GError **error)
 		guint32 entry_size_check;
 		guint32 crc;
 		guint32 crc_check;
+		TransactionFormat t_kind;
 
 		/* Check the end is not before where we currently are */
 		if (reader.current >= reader.end) {
-			g_set_error (error, TRACKER_DB_JOURNAL_ERROR, 0, 
-			             "End of journal reached");
+			/* Return FALSE as there is no further entry but
+			 * do not set error as it's not an error case.
+			 */
 			return FALSE;
 		}
 
@@ -863,7 +916,14 @@ tracker_db_journal_reader_next (GError **error)
 		reader.time = read_uint32 (reader.current);
 		reader.current += 4;
 
-		reader.type = TRACKER_DB_JOURNAL_START_TRANSACTION;
+		t_kind = read_uint32 (reader.current);
+		reader.current += 4;
+
+		if (t_kind == TRANSACTION_FORMAT_DATA)
+			reader.type = TRACKER_DB_JOURNAL_START_TRANSACTION;
+		else
+			reader.type = TRACKER_DB_JOURNAL_START_ONTOLOGY_TRANSACTION;
+
 		return TRUE;
 	} else if (reader.amount_of_triples == 0) {
 		/* end of transaction */
