@@ -20,12 +20,16 @@
 #ifndef __LIBTRACKER_MINER_MINER_FS_H__
 #define __LIBTRACKER_MINER_MINER_FS_H__
 
+#if !defined (__LIBTRACKER_MINER_H_INSIDE__) && !defined (TRACKER_COMPILATION)
+#error "Only <libtracker-miner/tracker-miner.h> can be included directly."
+#endif
+
 #include <glib-object.h>
 #include <gio/gio.h>
 
 #include <libtracker-client/tracker-sparql-builder.h>
 
-#include "tracker-miner.h"
+#include "tracker-miner-object.h"
 
 G_BEGIN_DECLS
 
