@@ -23,7 +23,7 @@
 #include <libtracker-common/tracker-type-utils.h>
 
 #include "tracker-crawler.h"
-#include "tracker-miner.h"
+#include "tracker-miner-object.h"
 #include "tracker-miner-manager.h"
 #include "tracker-marshal.h"
 #include "tracker-miner-client.h"
@@ -32,7 +32,7 @@
 /**
  * SECTION:tracker-miner-manager
  * @short_description: External control and monitoring of miners
- * @include: libtracker-miner/tracker-miner-manager.h
+ * @include: libtracker-miner/tracker-miner.h
  *
  * #TrackerMinerManager keeps track of available miners, their current
  * progress/status, and also allows basic external control on them, such
