@@ -43,13 +43,11 @@
 #include <enca.h>
 #endif
 
-#include <libtracker-common/tracker-date-time.h>
-#include <libtracker-common/tracker-file-utils.h>
-#include <libtracker-common/tracker-ontologies.h>
-#include <libtracker-common/tracker-utils.h>
+#include <libtracker-common/tracker-common.h>
+
+#include <libtracker-client/tracker.h>
 
 #include <libtracker-extract/tracker-extract.h>
-#include <libtracker-extract/tracker-utils.h>
 
 #include "tracker-albumart.h"
 #include "tracker-dbus.h"
@@ -234,9 +232,9 @@ static const struct {
 	{ "TCO", ID3V2_TCO },
 	{ "TCR", ID3V2_TCR },
 	{ "TLE", ID3V2_TLE },
-	{ "TPB", ID3V2_TPB },
 	{ "TP1", ID3V2_TP1 },
 	{ "TP2", ID3V2_TP2 },
+	{ "TPB", ID3V2_TPB },
 	{ "TT1", ID3V2_TT1 },
 	{ "TT2", ID3V2_TT2 },
 	{ "TT3", ID3V2_TT3 },
@@ -746,7 +744,7 @@ get_id3 (const gchar *data,
 	pos += 30;
 	year = g_convert (pos, 4, "UTF-8", encoding, NULL, NULL, NULL);
 	if (atoi (year) > 0) {
-		id3->recording_time = tracker_extract_guess_date (year);
+		id3->recording_time = tracker_date_guess (year);
 	}
 	g_free (year);
 
@@ -1239,11 +1237,11 @@ get_id3v24_tags (const gchar          *data,
 				tag->copyright = word;
 				break;
 			case ID3V24_TDRC:
-				tag->recording_time = tracker_extract_guess_date (word);
+				tag->recording_time = tracker_date_guess (word);
 				g_free (word);
 				break;
 			case ID3V24_TDRL:
-				tag->release_time = tracker_extract_guess_date (word);
+				tag->release_time = tracker_date_guess (word);
 				g_free (word);
 				break;
 			case ID3V24_TEXT:
@@ -1283,9 +1281,9 @@ get_id3v24_tags (const gchar          *data,
 				parts = g_strsplit (word, "/", 2);
 				if (parts[0]) {
 					tag->track_number = atoi (parts[0]);
-				}
-				if (parts[1]) {
-					tag->track_count = atoi (parts[1]);
+					if (parts[1]) {
+						tag->track_count = atoi (parts[1]);
+					}
 				}
 				g_strfreev (parts);
 				g_free (word);
@@ -1294,13 +1292,12 @@ get_id3v24_tags (const gchar          *data,
 			}
 			case ID3V24_TYER:
 				if (atoi (word) > 0) {
-					tag->recording_time = tracker_extract_guess_date (word);
+					tag->recording_time = tracker_date_guess (word);
 				}
 				g_free (word);
 				break;
 			default:
 				g_free (word);
-				g_warn_if_reached ();
 			}
 		}
 		}
@@ -1426,6 +1423,7 @@ get_id3v23_tags (const gchar          *data,
 
 			/* text frames */
 			word = id3v2_text_to_utf8 (data[pos], &data[pos + 1], csize - 1);
+
 			if (!tracker_is_empty_string (word)) {
 				g_strstrip (word);
 			}
@@ -1485,9 +1483,9 @@ get_id3v23_tags (const gchar          *data,
 				parts = g_strsplit (word, "/", 2);
 				if (parts[0]) {
 					tag->track_number = atoi (parts[0]);
-				}
-				if (parts[1]) {
-					tag->track_count = atoi (parts[1]);
+					if (parts[1]) {
+						tag->track_count = atoi (parts[1]);
+					}
 				}
 				g_strfreev (parts);
 				g_free (word);
@@ -1496,13 +1494,12 @@ get_id3v23_tags (const gchar          *data,
 			}
 			case ID3V24_TYER:
 				if (atoi (word) > 0) {
-					tag->recording_time = tracker_extract_guess_date (word);
+					tag->recording_time = tracker_date_guess (word);
 				}
 				g_free (word);
 				break;
 			default:
 				g_free (word);
-				g_warn_if_reached ();
 			}
 		}
 		}
@@ -1631,13 +1628,12 @@ get_id3v20_tags (const gchar          *data,
 				break;
 			case ID3V2_TYE:
 				if (atoi (word) > 0) {
-					tag->recording_time = tracker_extract_guess_date (word);
+					tag->recording_time = tracker_date_guess (word);
 				}
 				g_free (word);
 				break;
 			default:
 				g_free (word);
-				g_warn_if_reached ();
 			}
 		}
 
