@@ -33,10 +33,7 @@
 
 #include <totem-pl-parser.h>
 
-#include <libtracker-common/tracker-ontology.h>
-
 #include <libtracker-extract/tracker-extract.h>
-#include <libtracker-extract/tracker-utils.h>
 
 #define PLAYLIST_PROPERTY_NO_TRACKS "entryCounter"
 #define PLAYLIST_PROPERTY_DURATION  "listDuration"
@@ -48,11 +45,6 @@
 #define PLAYLIST_DEFAULT_NO_TRACKS 0
 #define PLAYLIST_DEFAULT_DURATION 0
 
-
-#define NFO_PREFIX TRACKER_NFO_PREFIX
-#define RDF_PREFIX TRACKER_RDF_PREFIX
-#define RDF_TYPE RDF_PREFIX "type"
-
 typedef struct {
 	guint32 track_counter;
 	gint64 total_time;
@@ -91,19 +83,6 @@ entry_parsed (TotemPlParser *parser, const gchar *to_uri, GHashTable *to_metadat
 	tracker_sparql_builder_predicate (data->metadata, "a");
 	tracker_sparql_builder_object (data->metadata, "nfo:MediaFileListEntry");
 
-	/* The DO for the entries is needed for writeback, which depends on nie:isStoredAs
-	 * being set. You can see it as "is stored in" instead of as ;) */
-
-	tracker_sparql_builder_predicate (data->metadata, "a");
-	tracker_sparql_builder_object (data->metadata, "nie:DataObject");
-	tracker_sparql_builder_predicate (data->metadata, "a");
-	tracker_sparql_builder_object (data->metadata, "nie:InformationElement");
-
-	tracker_sparql_builder_predicate (data->metadata, "nie:isStoredAs");
-	tracker_sparql_builder_object_iri (data->metadata, data->uri);
-	tracker_sparql_builder_predicate (data->metadata, "nie:url");
-	tracker_sparql_builder_object_string (data->metadata, data->uri);
-
 	tracker_sparql_builder_predicate (data->metadata, "nfo:entryContent");
 	tracker_sparql_builder_object_unvalidated (data->metadata, to_uri);
 
@@ -145,9 +124,6 @@ extract_playlist (const gchar          *uri,
 	tracker_sparql_builder_predicate (metadata, "a");
 	tracker_sparql_builder_object (metadata, "nfo:MediaList");
 
-	tracker_sparql_builder_predicate (metadata, "nie:isStoredAs");
-	tracker_sparql_builder_object_iri (metadata, uri);
-
 	result = totem_pl_parser_parse (pl, uri, FALSE);
 
 	switch (result) {
