@@ -29,18 +29,10 @@
 #include <vorbis/vorbisfile.h>
 
 #include <libtracker-common/tracker-file-utils.h>
-#include <libtracker-common/tracker-ontologies.h>
-#include <libtracker-common/tracker-utils.h>
+
+#include <libtracker-client/tracker.h>
 
 #include <libtracker-extract/tracker-extract.h>
-#include <libtracker-extract/tracker-utils.h>
-
-#define NIE_PREFIX TRACKER_NIE_PREFIX
-#define NFO_PREFIX TRACKER_NFO_PREFIX
-#define DC_PREFIX TRACKER_DC_PREFIX
-#define NCO_PREFIX TRACKER_NCO_PREFIX
-#define NMM_PREFIX TRACKER_NMM_PREFIX
-#define RDF_PREFIX TRACKER_RDF_PREFIX
 
 static void extract_vorbis (const char            *uri,
                             TrackerSparqlBuilder  *preupdate,
@@ -97,7 +89,7 @@ extract_vorbis (const char *uri,
 	gchar          *filename;
 	VorbisData      vorbis_data = { 0 };
 	VorbisNeedsMergeData merge_data = { 0 };
-	gchar *artist_uri = NULL, *album_uri = NULL, *publisher_uri = NULL;
+	gchar *artist_uri = NULL, *album_uri = NULL;
 
 	filename = g_filename_from_uri (uri, NULL, NULL);
 	f = tracker_file_open (filename, "r", FALSE);
@@ -127,7 +119,7 @@ extract_vorbis (const char *uri,
 		vorbis_data.AlbumGain = ogg_get_comment (comment, "AlbumGain");
 		vorbis_data.AlbumPeakGain = ogg_get_comment (comment, "AlbumPeakGain");
 		date = ogg_get_comment (comment, "date");
-		vorbis_data.date = tracker_extract_guess_date (date);
+		vorbis_data.date = tracker_date_guess (date);
 		g_free (date);
 		vorbis_data.comment = ogg_get_comment (comment, "comment");
 		vorbis_data.genre = ogg_get_comment (comment, "genre");
