@@ -27,7 +27,7 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
-#include <panel-applet-gconf.h>
+#include <panel-applet.h>
 
 #include "tracker-applet.h"
 #include "tracker-results-window.h"
@@ -83,18 +83,22 @@ applet_entry_start_search (TrackerApplet *applet)
 
 	text = gtk_entry_get_text (GTK_ENTRY (applet->entry));
 
+        if (!text || !*text) {
+                gtk_widget_hide (applet->results);
+                return;
+        }
+
 	g_print ("Searching for: '%s'\n", text);
 
 	if (!applet->results) {
-		/* gtk_widget_destroy (applet->results); */
-		/* applet->results = NULL; */
-
 		applet->results = tracker_results_window_new (applet->parent, text);
-
-		tracker_results_window_popup (TRACKER_RESULTS_WINDOW (applet->results));
 	} else {
 		g_object_set (applet->results, "query", text, NULL);
 	}
+
+	if (!GTK_WIDGET_VISIBLE (applet->results)) {
+		tracker_results_window_popup (TRACKER_RESULTS_WINDOW (applet->results));
+	}
 }
 
 static gboolean
@@ -133,6 +137,20 @@ applet_entry_button_press_event_cb (GtkWidget      *widget,
 	return FALSE;
 }
 
+static void
+applet_entry_editable_changed_cb (GtkWidget     *widget,
+                                  TrackerApplet *applet)
+{
+        if (applet->new_search_id) {
+                g_source_remove (applet->new_search_id);
+        }
+
+        applet->new_search_id =
+                g_timeout_add (300,
+                               applet_entry_start_search_cb,
+                               applet);
+}
+
 static gboolean
 applet_entry_key_press_event_cb (GtkWidget     *widget,
                                  GdkEventKey   *event,
@@ -151,15 +169,6 @@ applet_entry_key_press_event_cb (GtkWidget     *widget,
 		}
 
 		gtk_widget_grab_focus (applet->results);
-	} else {
-		if (applet->new_search_id) {
-			g_source_remove (applet->new_search_id);
-		}
-
-		applet->new_search_id =
-			g_timeout_add (300,
-			               applet_entry_start_search_cb,
-			               applet);
 	}
 
 	return FALSE;
@@ -216,6 +225,9 @@ applet_draw (gpointer user_data)
 	g_signal_connect (applet->entry,
 	                  "button_press_event",
 	                  G_CALLBACK (applet_entry_button_press_event_cb), applet);
+        g_signal_connect (applet->entry,
+                          "changed",
+                          G_CALLBACK (applet_entry_editable_changed_cb), applet);
 	g_signal_connect (applet->entry,
 	                  "key_press_event",
 	                  G_CALLBACK (applet_entry_key_press_event_cb), applet);
