@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
  * Copyright (C) 2008, Nokia
@@ -28,7 +27,7 @@
 #include <libtracker-common/tracker-log.h>
 #include <libtracker-common/tracker-utils.h>
 #include <libtracker-common/tracker-type-utils.h>
-#include <libtracker-common/tracker-ontology.h>
+#include <libtracker-common/tracker-ontologies.h>
 
 #include <libtracker-db/tracker-db-dbus.h>
 
@@ -41,6 +40,7 @@
 #include "tracker-resources.h"
 #include "tracker-resource-class.h"
 #include "tracker-events.h"
+#include "tracker-writeback.h"
 #include "tracker-store.h"
 
 #define RDF_PREFIX TRACKER_RDF_PREFIX
@@ -51,9 +51,13 @@ G_DEFINE_TYPE(TrackerResources, tracker_resources, G_TYPE_OBJECT)
 
 #define TRACKER_RESOURCES_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), TRACKER_TYPE_RESOURCES, TrackerResourcesPrivate))
 
+enum {
+	WRITEBACK,
+	LAST_SIGNAL
+};
 
 typedef struct {
-	GSList     *event_sources;
+	GSList *event_sources;
 } TrackerResourcesPrivate;
 
 typedef struct {
@@ -61,20 +65,23 @@ typedef struct {
 	guint request_id;
 } TrackerDBusMethodInfo;
 
+
+static guint signals[LAST_SIGNAL] = { 0 };
+
 static void
 free_event_sources (TrackerResourcesPrivate *priv)
 {
 	if (priv->event_sources) {
-		g_slist_foreach (priv->event_sources, 
-				 (GFunc) g_object_unref, NULL);
+		g_slist_foreach (priv->event_sources,
+		                 (GFunc) g_object_unref, NULL);
 		g_slist_free (priv->event_sources);
 
 		priv->event_sources = NULL;
 	}
 }
 
-static void 
-tracker_resources_finalize (GObject	 *object)
+static void
+tracker_resources_finalize (GObject      *object)
 {
 	TrackerResourcesPrivate *priv;
 
@@ -94,6 +101,16 @@ tracker_resources_class_init (TrackerResourcesClass *klass)
 
 	object_class->finalize = tracker_resources_finalize;
 
+	signals[WRITEBACK] =
+		g_signal_new ("writeback",
+		              G_OBJECT_CLASS_TYPE (object_class),
+		              G_SIGNAL_RUN_LAST,
+		              G_STRUCT_OFFSET (TrackerResourcesClass, writeback),
+		              NULL, NULL,
+		              g_cclosure_marshal_VOID__BOXED,
+		              G_TYPE_NONE, 1,
+		              TRACKER_TYPE_STR_STRV_MAP);
+
 	g_type_class_add_private (object_class, sizeof (TrackerResourcesPrivate));
 }
 
@@ -126,25 +143,26 @@ turtle_import_callback (GError *error, gpointer user_data)
 
 	if (error) {
 		tracker_dbus_request_failed (info->request_id,
-					     &error,
-					     NULL);
+		                             info->context,
+		                             &error,
+		                             NULL);
 		dbus_g_method_return_error (info->context, error);
 		return;
 	}
 
+	tracker_dbus_request_success (info->request_id,
+	                              info->context);
 	dbus_g_method_return (info->context);
-
-	tracker_dbus_request_success (info->request_id);
 }
 
 void
-tracker_resources_load (TrackerResources	 *object,
-			const gchar		 *uri,
-			DBusGMethodInvocation	 *context,
-			GError			**error)
+tracker_resources_load (TrackerResources         *object,
+                        const gchar              *uri,
+                        DBusGMethodInvocation    *context,
+                        GError                  **error)
 {
 	TrackerDBusMethodInfo   *info;
-	guint		    request_id;
+	guint               request_id;
 	GFile  *file;
 
 	request_id = tracker_dbus_get_next_request_id ();
@@ -152,9 +170,10 @@ tracker_resources_load (TrackerResources	 *object,
 	tracker_dbus_async_return_if_fail (uri != NULL, context);
 
 	tracker_dbus_request_new (request_id,
-				  "D-Bus request to load turtle file "
-				  "'%s'",
-				  uri);
+	                          context,
+	                          "%s(): uri:'%s'",
+	                          __FUNCTION__,
+	                          uri);
 
 	file = g_file_new_for_uri (uri);
 
@@ -170,14 +189,14 @@ tracker_resources_load (TrackerResources	 *object,
 }
 
 void
-tracker_resources_sparql_query (TrackerResources	 *self,
-				const gchar	         *query,
-				DBusGMethodInvocation	 *context,
-				GError			**error)
+tracker_resources_sparql_query (TrackerResources         *self,
+                                const gchar              *query,
+                                DBusGMethodInvocation    *context,
+                                GError                  **error)
 {
 	TrackerDBResultSet   *result_set;
-	GError 		     *actual_error = NULL;
-	guint		      request_id;
+	GError               *actual_error = NULL;
+	guint                 request_id;
 	GPtrArray            *values;
 
 	request_id = tracker_dbus_get_next_request_id ();
@@ -185,16 +204,18 @@ tracker_resources_sparql_query (TrackerResources	 *self,
 	tracker_dbus_async_return_if_fail (query != NULL, context);
 
 	tracker_dbus_request_new (request_id,
-				  "D-Bus request for SPARQL Query, "
-				  "query:'%s'",
-				  query);
+	                          context,
+	                          "%s(): '%s'",
+	                          __FUNCTION__,
+	                          query);
 
 	result_set = tracker_store_sparql_query (query, &actual_error);
 
 	if (actual_error) {
 		tracker_dbus_request_failed (request_id,
-					     &actual_error,
-					     NULL);
+		                             context,
+		                             &actual_error,
+		                             NULL);
 		dbus_g_method_return_error (context, actual_error);
 		g_error_free (actual_error);
 		return;
@@ -202,6 +223,7 @@ tracker_resources_sparql_query (TrackerResources	 *self,
 
 	values = tracker_dbus_query_result_to_ptr_array (result_set);
 
+	tracker_dbus_request_success (request_id, context);
 	dbus_g_method_return (context, values);
 
 	tracker_dbus_results_ptr_array_free (&values);
@@ -209,19 +231,17 @@ tracker_resources_sparql_query (TrackerResources	 *self,
 	if (result_set) {
 		g_object_unref (result_set);
 	}
-
-	tracker_dbus_request_success (request_id);
 }
 
 void
-tracker_resources_sparql_update (TrackerResources	 *self,
-				 const gchar	         *update,
-				 DBusGMethodInvocation	 *context,
-				 GError			**error)
+tracker_resources_sparql_update (TrackerResources        *self,
+                                 const gchar             *update,
+                                 DBusGMethodInvocation   *context,
+                                 GError                 **error)
 {
 	TrackerResourcesPrivate *priv;
-	GError 		     *actual_error = NULL;
-	guint		      request_id;
+	GError                       *actual_error = NULL;
+	guint                 request_id;
 
 	priv = TRACKER_RESOURCES_GET_PRIVATE (self);
 
@@ -230,24 +250,99 @@ tracker_resources_sparql_update (TrackerResources	 *self,
 	tracker_dbus_async_return_if_fail (update != NULL, context);
 
 	tracker_dbus_request_new (request_id,
-				  "D-Bus request for SPARQL Update, "
-				  "update:'%s'",
-				  update);
+	                          context,
+	                          "%s(): '%s'",
+	                          __FUNCTION__,
+	                          update);
 
 	tracker_store_sparql_update (update, &actual_error);
 
 	if (actual_error) {
 		tracker_dbus_request_failed (request_id,
-					     &actual_error,
-					     NULL);
+		                             context,
+		                             &actual_error,
+		                             NULL);
 		dbus_g_method_return_error (context, actual_error);
 		g_error_free (actual_error);
 		return;
 	}
 
+	tracker_dbus_request_success (request_id, context);
 	dbus_g_method_return (context);
+}
 
-	tracker_dbus_request_success (request_id);
+void
+tracker_resources_sparql_update_blank (TrackerResources       *self,
+                                       const gchar            *update,
+                                       DBusGMethodInvocation  *context,
+                                       GError                **error)
+{
+	TrackerResourcesPrivate *priv;
+	GError                       *actual_error = NULL;
+	guint                 request_id;
+	GPtrArray            *blank_nodes;
+
+	priv = TRACKER_RESOURCES_GET_PRIVATE (self);
+
+	request_id = tracker_dbus_get_next_request_id ();
+
+	tracker_dbus_async_return_if_fail (update != NULL, context);
+
+	tracker_dbus_request_new (request_id,
+	                          context,
+	                          "%s(): '%s'",
+	                          __FUNCTION__,
+	                          update);
+
+	blank_nodes = tracker_store_sparql_update_blank (update, &actual_error);
+
+	if (actual_error) {
+		tracker_dbus_request_failed (request_id,
+		                             context,
+		                             &actual_error,
+		                             NULL);
+		dbus_g_method_return_error (context, actual_error);
+		g_error_free (actual_error);
+		return;
+	}
+
+	if (!blank_nodes) {
+		/* Create empty GPtrArray for dbus-glib to be happy */
+		blank_nodes = g_ptr_array_new ();
+	}
+
+	tracker_dbus_request_success (request_id, context);
+	dbus_g_method_return (context, blank_nodes);
+
+	if (blank_nodes) {
+		gint i;
+
+		for (i = 0; i < blank_nodes->len; i++) {
+			g_ptr_array_foreach (blank_nodes->pdata[i], (GFunc) g_hash_table_unref, NULL);
+			g_ptr_array_free (blank_nodes->pdata[i], TRUE);
+		}
+		g_ptr_array_free (blank_nodes, TRUE);
+	}
+}
+
+void
+tracker_resources_sync (TrackerResources        *self,
+                        DBusGMethodInvocation   *context,
+                        GError                 **error)
+{
+	guint request_id;
+
+	request_id = tracker_dbus_get_next_request_id ();
+
+	tracker_dbus_request_new (request_id,
+	                          context,
+	                          "%s()",
+	                          __FUNCTION__);
+
+	tracker_data_sync ();
+
+	tracker_dbus_request_success (request_id, context);
+	dbus_g_method_return (context);
 }
 
 static void
@@ -257,27 +352,28 @@ batch_update_callback (GError *error, gpointer user_data)
 
 	if (error) {
 		tracker_dbus_request_failed (info->request_id,
-					     &error,
-					     NULL);
+		                             info->context,
+		                             &error,
+		                             NULL);
 		dbus_g_method_return_error (info->context, error);
 		return;
 	}
 
+	tracker_dbus_request_success (info->request_id,
+	                              info->context);
 	dbus_g_method_return (info->context);
-
-	tracker_dbus_request_success (info->request_id);
 }
 
 void
 tracker_resources_batch_sparql_update (TrackerResources          *self,
-				       const gchar	         *update,
-				       DBusGMethodInvocation	 *context,
-				       GError			**error)
+                                       const gchar               *update,
+                                       DBusGMethodInvocation     *context,
+                                       GError                   **error)
 {
 	TrackerDBusMethodInfo   *info;
 	TrackerResourcesPrivate *priv;
-	guint		      request_id;
-	gchar		      *sender;
+	guint                 request_id;
+	gchar                 *sender;
 
 	priv = TRACKER_RESOURCES_GET_PRIVATE (self);
 
@@ -286,9 +382,10 @@ tracker_resources_batch_sparql_update (TrackerResources          *self,
 	tracker_dbus_async_return_if_fail (update != NULL, context);
 
 	tracker_dbus_request_new (request_id,
-				  "D-Bus request for batch SPARQL Update, "
-				  "update:'%s'",
-				  update);
+	                          context,
+	                          "%s(): '%s'",
+	                          __FUNCTION__,
+	                          update);
 
 	info = g_slice_new (TrackerDBusMethodInfo);
 
@@ -308,26 +405,32 @@ batch_commit_callback (gpointer user_data)
 {
 	TrackerDBusMethodInfo *info = user_data;
 
+	tracker_data_sync ();
+
+	tracker_dbus_request_success (info->request_id,
+	                              info->context);
+
 	dbus_g_method_return (info->context);
-	tracker_dbus_request_success (info->request_id);
 }
 
 void
-tracker_resources_batch_commit (TrackerResources	 *self,
-				DBusGMethodInvocation	 *context,
-				GError			**error)
+tracker_resources_batch_commit (TrackerResources         *self,
+                                DBusGMethodInvocation    *context,
+                                GError                  **error)
 {
 	TrackerDBusMethodInfo *info;
 	TrackerResourcesPrivate *priv;
-	guint		      request_id;
-	gchar		      *sender;
+	guint                 request_id;
+	gchar                 *sender;
 
 	priv = TRACKER_RESOURCES_GET_PRIVATE (self);
 
 	request_id = tracker_dbus_get_next_request_id ();
 
 	tracker_dbus_request_new (request_id,
-				  "D-Bus request for batch commit");
+	                          context,
+	                          "%s()",
+	                          __FUNCTION__);
 
 	info = g_slice_new (TrackerDBusMethodInfo);
 
@@ -346,84 +449,115 @@ tracker_resources_batch_commit (TrackerResources	 *self,
 static void
 on_statements_committed (gpointer user_data)
 {
-	GPtrArray *events;
+	TrackerResources *resources = user_data;
+	GArray *events;
+	GHashTable *writebacks;
 	TrackerResourcesPrivate *priv;
 
-	priv = TRACKER_RESOURCES_GET_PRIVATE (user_data);
-
-	/* For more information about this call, look at the function end_batch
-	 * of tracker-store.c */
-	tracker_store_flush_journal ();
+	priv = TRACKER_RESOURCES_GET_PRIVATE (resources);
 
+	/* Class signals feature */
 	events = tracker_events_get_pending ();
 
 	if (events) {
-		GSList *event_sources, *l, *to_emit = NULL;
+		GSList *event_sources, *l;
 		guint i;
+		GHashTable *to_emit = NULL;
 
-		event_sources =priv->event_sources;
+		event_sources = priv->event_sources;
 
 		for (i = 0; i < events->len; i++) {
-			GValueArray *event = events->pdata[i];
-			const gchar *uri = g_value_get_string (g_value_array_get_nth (event, 0));
-			const gchar *predicate = g_value_get_string (g_value_array_get_nth (event, 1));
-			const gchar *rdf_class = g_value_get_string (g_value_array_get_nth (event, 2));
-			TrackerDBusEventsType type = g_value_get_int (g_value_array_get_nth (event, 3));
+			TrackerEvent *event;
+
+			event = &g_array_index (events, TrackerEvent, i);
 
 			for (l = event_sources; l; l = l->next) {
 				TrackerResourceClass *class_ = l->data;
-				if (g_strcmp0 (rdf_class, tracker_resource_class_get_rdf_class (class_)) == 0) {
-					tracker_resource_class_add_event (class_, uri, predicate, type);
-					to_emit = g_slist_prepend (to_emit, class_);
+				if (g_strcmp0 (tracker_class_get_uri (event->class), tracker_resource_class_get_rdf_class (class_)) == 0) {
+					tracker_resource_class_add_event (class_, event->subject, tracker_property_get_uri (event->predicate), event->type);
+					if (!to_emit) {
+						to_emit = g_hash_table_new (NULL, NULL);
+					}
+					g_hash_table_insert (to_emit, class_, class_);
 				}
 			}
 		}
 
 		if (to_emit) {
-			for (l = to_emit; l; l = l->next) {
-				TrackerResourceClass *class_ = l->data;
+			GHashTableIter iter;
+			gpointer key, value;
+
+			g_hash_table_iter_init (&iter, to_emit);
+
+			while (g_hash_table_iter_next (&iter, &key, &value)) {
+				TrackerResourceClass *class_ = key;
 				tracker_resource_class_emit_events (class_);
 			}
 
-			g_slist_free (to_emit);
+			g_hash_table_destroy (to_emit);
 		}
 	}
 
 	tracker_events_reset ();
+
+	/* Writeback feature */
+	writebacks = tracker_writeback_get_pending ();
+
+	if (writebacks) {
+		g_signal_emit (resources, signals[WRITEBACK], 0, writebacks);
+
+	}
+
+	tracker_writeback_reset ();
+}
+
+
+static void
+on_statements_rolled_back (gpointer user_data)
+{
+	tracker_events_reset ();
+	tracker_writeback_reset ();
 }
 
 static void
-on_statement_inserted (const gchar *subject,
-		       const gchar *predicate,
-		       const gchar *object,
-		       GPtrArray   *rdf_types,
-		       gpointer user_data)
+on_statement_inserted (const gchar *graph,
+                       const gchar *subject,
+                       const gchar *predicate,
+                       const gchar *object,
+                       GPtrArray   *rdf_types,
+                       gpointer user_data)
 {
 	if (g_strcmp0 (predicate, RDF_PREFIX "type") == 0) {
 		tracker_events_insert (subject, predicate, object, rdf_types, TRACKER_DBUS_EVENTS_TYPE_ADD);
 	} else {
 		tracker_events_insert (subject, predicate, object, rdf_types, TRACKER_DBUS_EVENTS_TYPE_UPDATE);
 	}
+
+	/* For predicates it's always update here */
+	tracker_writeback_check (graph, subject, predicate, object, rdf_types);
 }
 
 static void
-on_statement_deleted (const gchar *subject,
-		      const gchar *predicate,
-		      const gchar *object,
-		      GPtrArray   *rdf_types,
-		      gpointer user_data)
+on_statement_deleted (const gchar *graph,
+                      const gchar *subject,
+                      const gchar *predicate,
+                      const gchar *object,
+                      GPtrArray   *rdf_types,
+                      gpointer user_data)
 {
 	if (g_strcmp0 (predicate, RDF_PREFIX "type") == 0) {
 		tracker_events_insert (subject, predicate, object, rdf_types, TRACKER_DBUS_EVENTS_TYPE_DELETE);
 	} else {
 		tracker_events_insert (subject, predicate, object, rdf_types, TRACKER_DBUS_EVENTS_TYPE_UPDATE);
 	}
+
+	/* For predicates it's always delete here */
+	tracker_writeback_check (graph, subject, predicate, object, rdf_types);
 }
 
-
-void 
+void
 tracker_resources_prepare (TrackerResources *object,
-			   GSList           *event_sources)
+                           GSList           *event_sources)
 {
 	TrackerResourcesPrivate *priv;
 
@@ -431,9 +565,10 @@ tracker_resources_prepare (TrackerResources *object,
 
 	free_event_sources (priv);
 
-	tracker_data_set_insert_statement_callback (on_statement_inserted, object);
-	tracker_data_set_delete_statement_callback (on_statement_deleted, object);
-	tracker_data_set_commit_statement_callback (on_statements_committed, object);
+	tracker_data_add_insert_statement_callback (on_statement_inserted, object);
+	tracker_data_add_delete_statement_callback (on_statement_deleted, object);
+	tracker_data_add_commit_statement_callback (on_statements_committed, object);
+	tracker_data_add_rollback_statement_callback (on_statements_rolled_back, object);
 
 	priv->event_sources = event_sources;
 }
@@ -444,3 +579,4 @@ tracker_resources_unreg_batches (TrackerResources *object,
 {
 	tracker_store_unreg_batches (old_owner);
 }
+
