@@ -30,7 +30,7 @@
 #include <libtracker-db/tracker-db-dbus.h>
 #include <libtracker-db/tracker-db-manager.h>
 
-#include <libtracker-data/tracker-data-manager.h>
+#include <libtracker-data/tracker-data.h>
 
 #include "tracker-dbus.h"
 #include "tracker-marshal.h"
